/*
   InformixApi.c --
  
   This file incorporates the high level TCL-Informix-API-Interface

   This TCL-extension is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Written by Joachim Siebold <joachim.siebold@cityweb.de>
   based on DBD::Informix (Copyright J. Leffler and others)

   $Header: /home/cvs/informixtcl/InformixApi.c,v 1.2 2000/03/18 18:23:18 achim Exp $
   $Log: InformixApi.c,v $
   Revision 1.2  2000/03/18 18:23:18  achim
   Introduced CVS keywords

*/        


#include <stdio.h>
#include <string.h>
#include <tcl.h>
#include "InformixTcl.h"        

int rgetmsg _ANSI_ARGS_ ((int msgnum, char *s, int maxsize));

/*
 *--------------------------------------------------------------
 * 
 * InformixApiLogon
 *
 * Logs on to an Informix Database
 *
 * Results:
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 * Initializes logon handle data
 * Sets this connection to current connection
 *   
 *--------------------------------------------------------------
*/
int
InformixApiLogon(interp, logonVars, name, user, pass, conn_opt)
    Tcl_Interp      *interp;            /* Current Interpreter */
    LogonInfo       *logonVars;         /* Logon handle data */
    char            *name;              /* Database Name */
    char            *user;              /* Username */
    char            *pass;              /* Password */
    int             conn_opt;           /* Connection Option */

{
    int             autocommit;         /* Autocommit Option set */
    InformixSysInfo *SysInf;            /* Driver Info */
    static char     function[]=         "InformixApiLogon";
    static long     connection_num = 0;

    InformixApiEnter(function);
    autocommit=(conn_opt & INFORMIX_CONNOPT_AUTOCOMMIT);

    /* 
        Initialize Logon Handle Data
    */
    sprintf(logonVars->nm_connection, "x_%09ld", connection_num);      
    logonVars->interp= interp;
    logonVars->is_onlinedb= False;
    logonVars->is_loggeddb= False;
    logonVars->is_modeansi= False;
    logonVars->is_txactive= False;
    logonVars->is_connected= False;
    logonVars->has_autocommit= autocommit;
    logonVars->chop_blanks= False;
    connection_num++;

    /*
        Default User- and Databasename
    */
    if (name != 0 && *name == '\0')
        name = 0;
    if (name != 0 && strcmp(name, DEFAULT_DATABASE) == 0)
        name = 0;

    if(InformixIntLogon(logonVars,name,user,pass)==0) {
        InformixApiDebug(1,"Exit %s (**ERROR-1**)\n", function);
        return(TCL_ERROR);
    }
    /*
        Save sqlca for this database handle
    */
    InformixApiSaveSqlca(logonVars);

    /*
        Save this connection as current connection
    */
    SysInf= InformixTclGetSysinfo();
    SysInf->current_connection= logonVars;

    /*
        Save database name
    */
    if(name == (char *) NULL) {
        logonVars->database = (char *) ckalloc(1);
        strcpy(logonVars->database,"");
    } else {
        if((logonVars->database = (char *) ckalloc(strlen(name)+1))== NULL) {
            AppendResult (interp, "Allocation of memory failed");
            return(TCL_ERROR);
        }
        strcpy(logonVars->database, name);
    }

    /*
        Get and Save Database Type
    */
    logonVars->is_onlinedb = (sqlca.sqlwarn.sqlwarn3 == 'W');
    logonVars->is_modeansi = (sqlca.sqlwarn.sqlwarn2 == 'W');
    logonVars->is_loggeddb = (sqlca.sqlwarn.sqlwarn1 == 'W'); 
    logonVars->is_connected= True;

    /*
        Unlogged databases are in AutoCommit mode at all times and cannot be
        switched out of AutoCommit mode.
    */
    if (logonVars->is_loggeddb == False && logonVars->has_autocommit == False)
    {
        logonVars->has_autocommit = True;
    }

    /* 
        Start a transaction if the database is Logged 
        but not MODE ANSI and if AutoCommit is Off 
    */
    if (logonVars->is_loggeddb == True && logonVars->is_modeansi == False)
    {
        if ( logonVars->has_autocommit == False)
        {
            InformixApiBegin(interp,logonVars);
            if(sqlca.sqlcode < 0 ) {
                InformixApiSetError(interp, sqlca.sqlcode);
                InformixApiDebug(1,"Exit %s (**ERROR-3**)\n",function);
                return(TCL_ERROR);
            }
        }
    }                       

    InformixApiExit(function);
    return(TCL_OK);
}


/*
 *----------------------------------------------------------------------
 *
 * InformixApiLogoff
 *
 *      Closes connection to database, destroys any dependent statements
 *
 * Results:                     
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 *
 *----------------------------------------------------------------------
 */
int InformixApiLogoff (interp, logonVars)
    Tcl_Interp      *interp;            /* Current Interpreter */
    LogonInfo       *logonVars;         /* Logon handle data */

{
    InformixSysInfo *SysInf;            /* Driver Info */
    Tcl_HashSearch  searchPtr;          /* Hash Search Pointer */
    StatementInfo   *statementVars;     /* Statement Data */
    Tcl_HashEntry   *entry;             /* Hash Entries for lookup */
    static  char function[]="InformixApiLogoff";

    InformixApiEnter(function);
    /*
        Make given Logon Handle current connection
    */
    if(!InformixApiSetConnection(logonVars)) {
        InformixApiDebug(1,"%s -- set connection failed\n",function);
        return(TCL_ERROR);
    }

    /*
        Rollback Transactions
    */
    if (logonVars->is_loggeddb == True && logonVars->is_txactive == True) {
        InformixIntRollback(logonVars);
        if(sqlca.sqlcode < 0 ) {
            InformixApiSetError(interp, sqlca.sqlcode);
            InformixApiDebug(1,"%s -- rollback failed\n",function);
            return(TCL_ERROR);
        }
    }

    /*
        Close all Statement Handles belonging to this connection
    */
    InformixApiDebug(1,"%s -- deleting statement handles\n",function);
    SysInf= InformixTclGetSysinfo();
    entry= Tcl_FirstHashEntry(&SysInf->statementHash,&searchPtr);
    while(entry != NULL) {
        /*
            Get Statement Handle Data
        */
        
        if (InformixTclStatementHandle (interp, entry->key.string,
            &statementVars)!=TCL_OK) {
            return TCL_ERROR;
        }                               
        /*
            Does Statement belong to this connection ?
        */

        if(statementVars->logonVars == logonVars) {

            /*
                Invalidate statement
            */
            statementVars->magic= 0;

            /*
                Call the procedure which closes the statement
            */
            if(InformixApiClose (interp, statementVars)!= TCL_OK) 
                        return TCL_ERROR;

            if(InformixTclDeleteStatementHandle(interp,statementVars)
                        != TCL_OK) {
                return(TCL_ERROR);
            }
        }    
        entry=   Tcl_NextHashEntry(&searchPtr);
    }
    InformixApiDebug(1,"%s -- statement handles deleted\n",function);
    if (InformixIntLogoff(logonVars)==0) return TCL_ERROR;
    else return TCL_OK;

    InformixApiExit(function);
                                  
    logonVars->is_connected = False;
    SysInf->current_connection= (LogonInfo *) NULL;
    
    InformixApiExit(function);
    return(TCL_OK);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiDebug
 *
 *      Prints out Debug messages (on stderr) with string as argument
 *
 * Results:                     
 * none
 *
 *----------------------------------------------------------------------
 */
void InformixApiDebug(n, fmt, arg)
    int             n;                  /* Debug Level */
    char            *fmt;               /* Output Format */
    char            *arg;               /* Output Argument */
   
{
    InformixSysInfo *SysInf;            /* Driver Info */
    
    fflush(stderr);
    SysInf= InformixTclGetSysinfo();
    if(SysInf->debug >= n) fprintf(stderr,fmt,arg);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiDebugl
 *
 *      Prints out Debug messages (on stderr) with long as argument
 *
 * Results:                     
 * none
 *
 *----------------------------------------------------------------------
 */
void InformixApiDebugl(n, fmt, arg)
    int             n;                  /* Debug Level */
    char            *fmt;               /* Output Format */
    long            arg;                /* Output Argument */
   
{
    InformixSysInfo *SysInf;            /* Driver Info */
    
    fflush(stderr);
    SysInf= InformixTclGetSysinfo();
    if(SysInf->debug >= n) fprintf(stderr,fmt,arg);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiSetError
 *
 *      Format Informix Error Message (both SQL and ISAM Parts)
 *
 * Results:                     
 *      Sets the global TCL-Variables
 *          infmsg(errtxt)
 *          infmsg(errnum)
 *      Appends Message to TCL-Output
 *
 *----------------------------------------------------------------------
 */
void InformixApiSetError(interp,rc)
    Tcl_Interp      *interp;            /* Current Interpreter */
    ErrNum          rc;                 /* Error Number */

{
    char            errbuf[256];
    char            fmtbuf[256];
    char            sql_buf[256];
    char            isambuf[256];
    char            msgbuf[sizeof(sql_buf)+sizeof(isambuf)];

    if (rc < 0 ) {
        /* Format SQL (primary) error */
        if (rgetmsg(rc, errbuf, sizeof(errbuf)) != 0)
            strcpy(errbuf, "<<Failed to locate SQL error message>>");
        sprintf(fmtbuf, errbuf, sqlca.sqlerrm);
        sprintf(sql_buf, "SQL: %ld: %s", rc, fmtbuf);
        sql_buf[strlen(sql_buf)-1]='\0';
        strcpy(msgbuf, sql_buf);

        /* Format ISAM (secondary) error */  
        if (sqlca.sqlerrd[1] != 0)
        {
            if (rgetmsg(sqlca.sqlerrd[1], errbuf, sizeof(errbuf)) != 0)
                strcpy(errbuf, "<<Failed to locate ISAM error message>>");
            sprintf(fmtbuf, errbuf, sqlca.sqlerrm);
            sprintf(isambuf, "ISAM: %ld: %s", sqlca.sqlerrd[1], fmtbuf);
            isambuf[strlen(isambuf)-1]='\0';
            strcat(msgbuf,"\n");
            strcat(msgbuf,isambuf);
        }

        /* Record error number, error message, and error state */
        InformixSetMsgString(interp,InformixErrortxt,msgbuf);
        InformixSetMsgInt(interp,InformixErrornum,rc);
        /*
            Output Error Message
        */
        InformixApiDebug(6,"Error Message= %s\n",msgbuf);
        AppendResult (interp,msgbuf);
    }
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiSetError2
 *
 *      Generate Error Message
 *
 * Results:                     
 *      Appends Message to TCL-Output
 *
 *----------------------------------------------------------------------
 */
void InformixApiSetError2(logonVars,msg)
    LogonInfo       *logonVars;          /* Logon Handle */
    char            *msg;               /* Error Message */

{
    AppendResult(logonVars->interp,msg);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiEnter
 *
 *      Debug printout enter function
 *
 * Results:                     
 *
 *----------------------------------------------------------------------
 */
void InformixApiEnter(func)
    char            *func;      /* function name */
{
    InformixApiDebug(1,"Enter %s ()\n",func);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiExit
 *
 *      Debug printout exit function
 *
 * Results:                     
 *
 *----------------------------------------------------------------------
 */
void InformixApiExit(func)
    char            *func;      /* function name */
{
    InformixApiDebug(1,"Exit %s ()\n",func);
}
                                
/*
 *----------------------------------------------------------------------
 *
 * InformixApiSaveSqlca
 *
 *      Saves current sqlca record
 *
 * Results:                     
 *
 *----------------------------------------------------------------------
 */
void InformixApiSaveSqlca(logonVars)
    LogonInfo       *logonVars;     /* logon handle */

{
    logonVars->ix_sqlca= sqlca;
}

/*
 *----------------------------------------------------------------------
 *
 * InformixApiSqlCode
 *
 *      Record (and report) and SQL error, saving SQLCA information
 *
 * Results:                     
 *
 *----------------------------------------------------------------------
 */
void InformixApiSqlCode(logonVars)
    LogonInfo       *logonVars;         /* logon handle */

{
    if (sqlca.sqlcode < 0)
    {
        InformixApiSaveSqlca(logonVars);
        InformixApiSetError(logonVars->interp,sqlca.sqlcode);
    }                                   
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiSetConnection
 *
 *      Ensure that the correct connection is current
 *
 * Results:                     
 *
 *----------------------------------------------------------------------
 */
int InformixApiSetConnection(logonVars)
    LogonInfo       *logonVars;     /* logon handle */

{
    int             rc;             /* Return value */
    InformixSysInfo *SysInf;        /* Driver Info */

    
    /* 
        If this connection isn't connected, return with failure 
        Primarily a concern when destroying connections 
     */
    if (logonVars->is_connected == False)
        return(TCL_ERROR);

    rc= True;
    SysInf= InformixTclGetSysinfo();
    if (logonVars != SysInf->current_connection ) {
        InformixIntSetConnection(logonVars->nm_connection);
        if(sqlca.sqlcode < 0 ) {
            rc= False;
            InformixApiSqlCode(logonVars);
        }
        else SysInf->current_connection= logonVars;
    }
    return(rc);
}

/*
 *----------------------------------------------------------------------
 *
 * InformixApiBegin
 *
 *      Begins Transaction, logonVars becomes current connection
 *
 * Results:                     
 * TCL_OK on success, TCL_ERROR otherwise
 *
 *----------------------------------------------------------------------
 */
int InformixApiBegin(interp,logonVars)
    Tcl_Interp      *interp;            /* Current Interpreter */
    LogonInfo       *logonVars;     /* logon handle */
{
    int             rc;             /* Return Value */

    rc= TCL_OK;
    if(logonVars-> is_loggeddb) {
        if(!InformixApiSetConnection(logonVars)) {
            return(TCL_ERROR);
        }
        if(!InformixIntBegin(logonVars)) rc= TCL_ERROR;
        InformixApiSqlCode(logonVars);
    }
    return(rc);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiCommit
 *
 *      Commits Transaction, logonVars becomes current connection
 *
 * Results:                     
 * TCL_OK on success, TCL_ERROR otherwise
 *
 *----------------------------------------------------------------------
 */
int InformixApiCommit(interp,logonVars)
    Tcl_Interp      *interp;            /* Current Interpreter */
    LogonInfo       *logonVars;     /* logon handle */
{
    int             rc;             /* Return Value */
    int             rc2;            /* Return Value of Commit */

    rc= TCL_OK;
    if(logonVars-> is_loggeddb) {
        if(!InformixApiSetConnection(logonVars)) {
            return(TCL_ERROR);
        }
        rc2=InformixIntCommit(logonVars);
        InformixApiSqlCode(logonVars);
        if(rc2) {
            if(logonVars->is_modeansi == False &&
                logonVars->has_autocommit == False) {
                if(!InformixIntBegin(logonVars)) rc= TCL_ERROR;
                InformixApiSqlCode(logonVars);
            }
        } else {
            rc= TCL_ERROR;
        }
    }
    return(rc);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiRollback
 *
 *      Rollbacks Transaction, logonVars becomes current connection
 *
 * Results:                     
 * TCL_OK on success, TCL_ERROR otherwise
 *
 *----------------------------------------------------------------------
 */
int InformixApiRollback(interp,logonVars)
    Tcl_Interp      *interp;            /* Current Interpreter */
    LogonInfo       *logonVars;     /* logon handle */
{
    int             rc;             /* Return Value */
    int             rc2;            /* Return Value of Rollback */

    rc= TCL_OK;
    if(logonVars-> is_loggeddb) {
        if(!InformixApiSetConnection(logonVars)) {
            return(TCL_ERROR);
        }
        rc2=InformixIntRollback(logonVars);
        InformixApiSqlCode(logonVars);
        if(rc2) {
            if(logonVars->is_modeansi == False &&
                logonVars->has_autocommit == False) {
                if(!InformixIntBegin(logonVars)) rc= TCL_ERROR;
                InformixApiSqlCode(logonVars);
            }
        } else {
            rc= TCL_ERROR;
        }
    }
    return(rc);
}
/*
 *--------------------------------------------------------------
 *
 * InformixApiPrepare
 *
 * Prepares a new Statement
 *
 * Results:
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 * Initializes statement handle data
 *
 *--------------------------------------------------------------
*/
int
InformixApiPrepare(interp, statementVars, cmd)
    Tcl_Interp      *interp;            /* Current Interpreter */
    StatementInfo   *statementVars;     /* Statement handle data */
    char            *cmd;               /* Statement */

{
    static long     cursor_num=0;       /* Cursor Name counter */
    static char     function[]=         "InformixApiPrepare";
    int             rc=TCL_OK;          /* Return Value */

    InformixApiEnter(function);
    /*
        Initialize Statement Data Structure
    */
    sprintf(statementVars->nm_stmnt, "p_%09ld", cursor_num);
    sprintf(statementVars->nm_cursor, "c_%09ld", cursor_num);
    sprintf(statementVars->nm_obind, "d_%09ld", cursor_num);
    sprintf(statementVars->nm_ibind, "b_%09ld", cursor_num);
    statementVars->st_state = Unused;
    statementVars->st_type = 0;
    statementVars->n_blobs = 0;
    statementVars->n_bound = 0;
    statementVars->n_rows = 0;
    statementVars->n_columns = 0;                   
    statementVars->blob_bind = BLOB_IN_MEMORY;                   
    cursor_num++;

    /*
        Make given Logon Handle current connection
    */
    if(!InformixApiSetConnection(statementVars->logonVars)) {
        InformixApiDebug(1,"%s -- set connection failed\n",function);
        return(TCL_ERROR);
    }   

    /*
        Prepare Statement
    */
    if(InformixIntPrepare(statementVars,cmd)==0) rc=TCL_ERROR;

    InformixApiExit(function);
    return(rc);
}

/*
 *--------------------------------------------------------------
 *
 * InformixApiClose
 *
 * Closes a Statement
 *
 * Results:
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 *
 *--------------------------------------------------------------
*/
int
InformixApiClose(interp, statementVars)
    Tcl_Interp      *interp;            /* Current Interpreter */
    StatementInfo   *statementVars;     /* Statement handle data */

{
    static char     function[]=         "InformixApiClose";
    int             rc=TCL_OK;          /* Return Value */

    InformixApiEnter(function);
    /*
        Make given Logon Handle current connection
    */
    if(!InformixApiSetConnection(statementVars->logonVars)) {
        InformixApiDebug(1,"%s -- set connection failed\n",function);
        return(TCL_ERROR);
    }   
    /*
        Close Cursor
    */
    if(InformixIntClose(statementVars)==0)rc=TCL_ERROR;
    InformixApiExit(function);
    return(rc);
}

/*
 *--------------------------------------------------------------
 *
 * InformixApiExecute
 *
 * Executes prepared Statement, binds Parameters
 *
 * Results:
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 *
 *--------------------------------------------------------------
*/
int
InformixApiExecute(interp, statementVars, objc, objv)
    Tcl_Interp      *interp;            /* Current Interpreter */
    StatementInfo   *statementVars;     /* Statement Handle */ 
    int             objc;               /* Number of arguments. */
    Tcl_Obj         *CONST objv[];      /* Argument objects. */      

{
    static char     function[]=         "InformixApiExecute";

    int rc;
    int rv;
    int i;

    InformixApiEnter(function);
    if ((rc = InformixApiSetConnection(statementVars->logonVars)) == 0)
    {
        InformixApiSqlCode(statementVars->logonVars);
        InformixApiExit(function);
        return TCL_ERROR;
    }
    /*
        Does the number of parameters match the number of bind variables?
    */
    if(objc != statementVars->n_bound) {
        AppendResult(statementVars->logonVars->interp,
            "Number of parameters does not match the number of bind variables");
        return TCL_ERROR;
    }

    /*
        Parameter binding 
    */
    for(i=0; i< objc; i++) {
        if (InformixIntBind(statementVars,i,objv[i])==0) {
            InformixApiSqlCode(statementVars->logonVars);
            InformixApiExit(function);
            return TCL_ERROR;
        }
    }

    if (statementVars->st_type == SQ_SELECT)
        rc = InformixIntOpen(statementVars);    
#ifdef SQ_EXECPROC
    else if (statementVars->st_type == SQ_EXECPROC && 
            statementVars->n_columns > 0)
        rc = InformixIntOpen(statementVars);
#endif /* SQ_EXECPROC */
    else
        rc = InformixIntExec(statementVars);

    /* Map returned values from Open and Exec functions */
    if (rc == 0)
    {
        /* Statement failed -- return the error code */
        InformixApiExit(function);
        return TCL_ERROR;
    }
    else
    {
        /*
         * Statement succeeded.  Don't forget about MODE ANSI database and
         * an UPDATE which does not alter any rows returning SQLNOTFOUND.
         * MODE ANSI problem found by Chuck.Collins@zool.Airtouch.com
         */
        rv = sqlca.sqlerrd[2];                        
        /*** Number of cols? ***/
    }

    InformixApiExit(function);
    return TCL_OK;
}      
/*
 *--------------------------------------------------------------
 *
 * InformixApiFetch
 *
 * Fetches from cursor
 *
 * Results:
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 *
 *--------------------------------------------------------------
*/
int
InformixApiFetch(interp, statementVars, objc, objv)
    Tcl_Interp      *interp;            /* Current Interpreter */
    StatementInfo   *statementVars;     /* Statement Handle */ 
    int             objc;               /* Number of Variables */
    Tcl_Obj         *CONST objv[];      /* Variable Names */
{
    Tcl_Obj         *resList;           /* Result List */

    /*
        Make connection current connection
    */
    if (InformixApiSetConnection(statementVars->logonVars)==0) {
        InformixApiSaveSqlca(statementVars->logonVars);
        return TCL_ERROR;
    }         

    /*
        Check number of Variables, ignore errors if statement in wrong state
    */
    if(statementVars->st_state == Opened) {
        if((objc>0) && (objc != statementVars->n_columns)) {
            AppendResult(interp,
                "Number of variables does not match number of columns");
            return(TCL_ERROR);
        }
    }
    /*
        If objc is zero then we have to allocate a new Object for the 
        result list.
    */
    if(objc==0) {
        Tcl_ResetResult(statementVars->logonVars->interp);
        resList= Tcl_NewListObj(0, (Tcl_Obj **) NULL);
        Tcl_SetObjResult(statementVars->logonVars->interp, resList);
    } else {
        resList= (Tcl_Obj *) NULL;
    }
    /*
        Fetch Row
    */
    if(InformixIntFetch(statementVars,objc,objv,resList)==0) return TCL_ERROR;
    else return(TCL_OK);
}


/*
 *--------------------------------------------------------------
 *
 * InformixApiRelease
 *
 * Releases a statement Handle with all Ressources
 *
 * Results:
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 *
 *--------------------------------------------------------------
*/
int
InformixApiRelease(interp, statementVars)
    Tcl_Interp      *interp;            /* Current Interpreter */
    StatementInfo   *statementVars;     /* Statement handle data */

{
    static char     function[]=         "InformixApiRelease";
    int             rc=TCL_OK;          /* Return Value */

    InformixApiEnter(function);
    /*
        Make given Logon Handle current connection
    */
    if(!InformixApiSetConnection(statementVars->logonVars)) {
        InformixApiDebug(1,"%s -- set connection failed\n",function);
        return(TCL_ERROR);
    }   
    /*
        Release Statement
    */
    if(InformixIntRelease(statementVars)==0)rc=TCL_ERROR;
    InformixApiExit(function);
    return(rc);
}
/*
 *--------------------------------------------------------------
 *
 * InformixApiGetAttrib
 *
 * Gets Column Attributes
 *
 * Results:
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 *
 *--------------------------------------------------------------
*/
int
InformixApiGetAttrib(interp, statementVars, itype)
    Tcl_Interp      *interp;            /* Current Interpreter */
    StatementInfo   *statementVars;     /* Statement handle data */
    int             itype;              /* Type of Information */

{
    int             i;
    char            strval[NAMESIZE];
    long            longval;
    int             rc=1;
    Tcl_Obj         *resultPtr;         /* current interps result */

    /*
        Make connection current connection
    */
    if (InformixApiSetConnection(statementVars->logonVars)==0) {
        InformixApiSaveSqlca(statementVars->logonVars);
        return TCL_ERROR;
    }         
    resultPtr= Tcl_GetObjResult(interp); 
    for(i=1;i<= statementVars->n_columns; i++) {
        rc=InformixIntGetAttrib(statementVars,i,itype,&longval,strval);
        if(rc==0) return TCL_ERROR;
        switch(itype) {
        case 0:
        case 2:
            Tcl_ListObjAppendElement(interp, resultPtr,
                        Tcl_NewStringObj(strval,-1));  
            break;
        case 1:
        case 3:
        case 4:
        case 5:
            Tcl_ListObjAppendElement(interp, resultPtr,
                        Tcl_NewLongObj(longval));  
            break;
        }
    }
    return(TCL_OK);
}
/*
 *----------------------------------------------------------------------
 *
 * InformixApiDisconnectAll
 *
 *      Closes every connection to the Informix Database
 *
 * Results:                     
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 *
 *----------------------------------------------------------------------
 */
int InformixApiDisconnectAll (interp)
    Tcl_Interp      *interp;            /* Current Interpreter */

{
    InformixSysInfo *SysInf;            /* Driver Info */
    Tcl_HashSearch  searchPtr;          /* Hash Search Pointer */
    LogonInfo       *logonVars;         /* Logon Data */
    Tcl_HashEntry   *entry;             /* Hash Entries for lookup */
    int             rc;
    static  char function[]="InformixApiDisconnectAll";

    InformixApiEnter(function);

    /*
        Close all Logon Handles belonging to this interpreter
    */
    InformixApiDebug(1,"%s -- deleting logon handles\n",function);
    SysInf= InformixTclGetSysinfo();
    entry= Tcl_FirstHashEntry(&SysInf->logonHash,&searchPtr);
    while(entry != NULL) {
        /*
            Get Logon Handle Data
        */
        
        if (InformixTclLogonHandle (interp, entry->key.string,
            &logonVars)!=TCL_OK) {
            return TCL_ERROR;
        }                               
        /*
            Invalidate Logon Handle
        */
        logonVars-> magic=0,
        rc= InformixApiLogoff(interp,logonVars);
        if(rc== TCL_ERROR) {
            InformixApiExit(function);
            return(TCL_ERROR);
        }

        /*
            Delete Hash Entry
        */
        if (InformixTclDeleteLogonHandle(interp,logonVars)!= TCL_OK)
            return TCL_ERROR;
        entry=   Tcl_NextHashEntry(&searchPtr);
    }
    InformixApiExit(function);
    return(TCL_OK);
}
/*
 *--------------------------------------------------------------
 *
 * InformixApiDosql
 *
 * Executes a statement (With execute immediate)
 *
 * Results:
 * Returns TCL_OK on success, TCL_ERROR otherwise.
 *
 *--------------------------------------------------------------
*/
int
InformixApiDosql(interp, logonVars, cmd)
    Tcl_Interp      *interp;            /* Current Interpreter */
    LogonInfo       *logonVars;         /* Logon handle data */
    char            *cmd;               /* Statement */

{
    static char     function[]=         "InformixApiDosql";
    int             rc=TCL_OK;          /* Return Value */

    InformixApiEnter(function);

    /*
        Make given Logon Handle current connection
    */
    if(!InformixApiSetConnection(logonVars)) {
        InformixApiDebug(1,"%s -- set connection failed\n",function);
        return(TCL_ERROR);
    }   

    /*
        Execute Statement
    */
    if(InformixIntDosql(logonVars,cmd)==0) rc=TCL_ERROR;

    InformixApiExit(function);
    return(rc);
}

